import { Collection } from '../../../lib/src/collection/Collection.ts'

/**
 * A single query row, as an object.
 */
export type QueryRow = { [key: string]: any }

/**
 * A valid key on a model.
 */
export type ModelKey = string | number

/**
 * Interface for the result of a query execution.
 */
export interface QueryResult {
    rows: Collection<QueryRow>,
    row_count: number,
}

/**
 * Database column types.
 */
export enum Type {
    bigint = 'bigint',
    int8 = 'bigint',
    bigserial = 'bigserial',
    serial8 = 'bigserial',
    bit = 'bit',
    bit_varying = 'bit varying',
    varbit = 'bit varying',
    boolean = 'boolean',
    bool = 'boolean',
    box = 'box',
    bytea = 'bytea',
    character = 'character',
    char = 'character',
    character_varying = 'character varying',
    varchar = 'character varying',
    cidr = 'cidr',
    circle = 'circle',
    date = 'date',
    double_precision = 'double precision',
    float8 = 'double precision',
    inet = 'inet',
    integer = 'integer',
    int = 'integer',
    int4 = 'integer',
    interval = 'interval',
    json = 'json',
    line = 'line',
    lseg = 'lseg',
    macaddr = 'macaddr',
    money = 'money',
    numeric = 'numeric',
    decimal = 'numeric',
    path = 'path',
    point = 'point',
    polygon = 'polygon',
    real = 'real',
    float4 = 'real',
    smallint = 'smallint',
    int2 = 'smallint',
    smallserial = 'smallserial',
    serial2 = 'smallserial',
    serial = 'serial',
    serial4 = 'serial',
    text = 'text',
    time = 'time',
    timestamp = 'timestamp',
    tsquery = 'tsquery',
    tsvector = 'tsvector',
    txid_snapshot = 'txid_snapshot',
    uuid = 'uuid',
    xml = 'xml',
    other = 'other',
}
