import {Builder} from '../../builder/Builder.ts'
import {Select} from '../../builder/type/Select.ts'
import {Relation} from './Relation.ts'
import {Update} from '../../builder/type/Update.ts'
import {FieldSet, QuerySource} from '../../builder/types.ts'
import {Delete} from '../../builder/type/Delete.ts'
import {Model} from '../Model.ts'

/**
 * Query builder scoped to relation values.
 * @extends Builder
 */
export class RelationBuilder<T extends Model<T>> extends Builder<T> {
    constructor(
        /**
         * The relation whose related model should be the subject of these queries.
         * @type Relation
         */
        protected relation: Relation<any, T>
    ) {
        super()
    }

    public select(...fields: FieldSet[]): Select<T> {
        const select = this.relation.related.select(...fields)
            .from(this.relation.related_query_source)

        select.target_operator(this.relation.get_operator())
        this.relation.scope_query(select)
        return select
    }

    public update(target?: QuerySource, alias?: string): Update<T> {
        const update = this.relation.related.update(this.relation.related_query_source)

        update.target_operator(this.relation.get_operator())
        this.relation.scope_query(update)
        return update
    }

    public delete(target?: QuerySource, alias?: string): Delete<T> {
        const delete_query = this.relation.related.delete(this.relation.related_query_source)

        delete_query.target_operator(this.relation.get_operator())
        this.relation.scope_query(delete_query)
        return delete_query
    }
}
